# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
from azure.cli.testsdk import (ScenarioTest, JMESPathCheck, ResourceGroupPreparer,
                               StorageAccountPreparer, api_version_constraint, record_only)
from azure.cli.core.profiles import ResourceType
from ..storage_test_util import StorageScenarioMixin


@record_only()
@api_version_constraint(ResourceType.DATA_STORAGE, min_api='2017-11-09')
class StorageOauthTests(StorageScenarioMixin, ScenarioTest):
    def oauth_cmd(self, cmd, *args, **kwargs):
        return self.cmd(cmd + ' --auth-mode login', *args, **kwargs)

    @ResourceGroupPreparer(name_prefix='cli_test_storage_oauth')
    @StorageAccountPreparer()
    def test_storage_blob_oauth(self, resource_group, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'account': storage_account,
            'container': self.create_random_name(prefix='container', length=20),
            'local_dir': self.create_temp_dir(),
            'local_file': self.create_temp_file(1),
            'blob': self.create_random_name(prefix='blob', length=20)
        })

        self.oauth_cmd('storage container create -n {container} --account-name {account}', checks=[
            JMESPathCheck('created', True)])

        # # upload a blob
        self.oauth_cmd('storage blob upload -c {container} -f "{local_file}" -n {blob} --type block '
                       '--account-name {account}')
        self.oauth_cmd('storage blob exists -n {blob} -c {container} --account-name {account}', checks=[
            JMESPathCheck('exists', True)])
        self.oauth_cmd('storage blob list -c {container} --account-name {account}', checks=[
            JMESPathCheck('length(@)', 1)])
        self.oauth_cmd('storage blob show -c {container} --account-name {account} -n {blob}', checks=[
            JMESPathCheck('name', self.kwargs['blob'])])

        # download the blob
        self.kwargs['download_path'] = os.path.join(self.kwargs.get('local_dir'), 'test.file')
        self.oauth_cmd('storage blob download -n {blob} -c {container} --file "{download_path}"'
                       ' --account-name {account}')

    @ResourceGroupPreparer(name_prefix='cli_test_storage_oauth')
    @StorageAccountPreparer()
    def test_storage_queue_oauth(self, resource_group, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'account': storage_account,
            'queue': self.create_random_name(prefix='queue', length=20)
        })

        self.oauth_cmd('storage queue create -n {queue} --account-name {account} --fail-on-exist --metadata a=b c=d',
                       checks=[JMESPathCheck('created', True)])
        self.oauth_cmd('storage queue exists -n {queue} --account-name {account}', checks=[
            JMESPathCheck('exists', True)])

        res = self.oauth_cmd('storage queue list --account-name {account}').get_output_in_json()
        self.assertIn(self.kwargs.get('queue'), [x['name'] for x in res], 'The newly created queue is not listed.')

        self.oauth_cmd('storage queue metadata show -n {queue} --account-name {account}', checks=[
            JMESPathCheck('a', 'b'),
            JMESPathCheck('c', 'd')
        ])

        self.oauth_cmd('storage queue metadata update -n {queue} --account-name {account} --metadata e=f g=h')
        self.oauth_cmd('storage queue metadata show -n {queue} --account-name {account}', checks=[
            JMESPathCheck('e', 'f'),
            JMESPathCheck('g', 'h')
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
import unittest

_running_state = "Running"
_stopped_state = "Stopped"


class AzureKustoClusterTests(ScenarioTest):

    @ResourceGroupPreparer()
    def test_kusto_cluster_life_cycle(self, resource_group):
        self.kwargs.update({
            'sku': 'D13_v2',
            'name': self.create_random_name(prefix='test', length=20),
            'location': "Central US",
            'capacity': 4
        })

        # Create cluster
        self.cmd('az kusto cluster create -n {name} -g {rg} --sku {sku} --capacity {capacity}',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _running_state),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('sku.capacity', '{capacity}')])

        # Get cluster
        self.cmd('az kusto cluster show -n {name} -g {rg}',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _running_state),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('sku.capacity', '{capacity}')])

        # Update cluster
        self.kwargs.update({
            'sku': 'D14_v2',
            'capacity': 6
        })
        self.cmd('az kusto cluster update -n {name} -g {rg} --sku {sku} --capacity {capacity} ',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _running_state),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('sku.capacity', '{capacity}')])

        # Delete cluster
        self.cmd('az kusto cluster delete -n {name} -g {rg} -y')

    @ResourceGroupPreparer()
    def test_kusto_cluster_stop_start(self, resource_group):
        self.kwargs.update({
            'sku': 'D13_v2',
            'name': self.create_random_name(prefix='test', length=20),
        })

        self.cmd('az kusto cluster create -n {name} -g {rg} --sku {sku}',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _running_state),
                         self.check('provisioningState', 'Succeeded')])

        self.cmd('az kusto cluster stop -n {name} -g {rg}')

        # Verify that the state is stopped
        self.cmd('az kusto cluster show -n {name} -g {rg}',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _stopped_state),
                         self.check('provisioningState', 'Succeeded')])

        self.cmd('az kusto cluster start -n {name} -g {rg}')

        # Verify that the state is running
        self.cmd('az kusto cluster show -n {name} -g {rg}',
                 checks=[self.check('name', '{name}'),
                         self.check('sku.name', '{sku}'),
                         self.check('state', _running_state),
                         self.check('provisioningState', 'Succeeded')])

        self.cmd('az kusto cluster delete -n {name} -g {rg} -y')


class AzureKustoDatabaseTests (ScenarioTest):
    @ResourceGroupPreparer()
    def test_kusto_database_life_cycle(self, resource_group):
        self.kwargs.update({
            'sku': 'D13_v2',
            'cluster_name': self.create_random_name(prefix='test', length=20),
            'database_name': self.create_random_name(prefix='testdb', length=20),
            'location': "Central US",
            'soft_delete_period': 'P10D',
            'hot_cache_period': 'P5D',
            'soft_delete_period_display': '10 days, 0:00:00',
            'hot_cache_period_display': '5 days, 0:00:00'
        })

        # Create cluster
        self.cmd('az kusto cluster create -n {cluster_name} -g {rg} --sku {sku}',
                 checks=[self.check('name', '{cluster_name}'),
                         self.check('sku.name', '{sku}')])

        # Create database
        self.cmd('az kusto database create --cluster-name {cluster_name} -g {rg} -n {database_name}  --soft-delete-period {soft_delete_period} --hot-cache-period {hot_cache_period}',
                 checks=[self.check('name', '{cluster_name}/{database_name}'),
                         self.check('softDeletePeriod', '{soft_delete_period_display}'),
                         self.check('hotCachePeriod', '{hot_cache_period_display}')])

        # Update database
        self.kwargs.update({
            'soft_delete_period': 'P20D',
            'hot_cache_period': 'P10D',
            'soft_delete_period_display': '20 days, 0:00:00',
            'hot_cache_period_display': '10 days, 0:00:00'
        })

        self.cmd('az kusto database update --cluster-name {cluster_name} -g {rg} -n {database_name}  --soft-delete-period {soft_delete_period} --hot-cache-period {hot_cache_period}',
                 checks=[self.check('name', '{cluster_name}/{database_name}'),
                         self.check('softDeletePeriod', '{soft_delete_period_display}'),
                         self.check('hotCachePeriod', '{hot_cache_period_display}')])

        # Delete database
        self.cmd('az kusto database delete --cluster-name {cluster_name} -g {rg} -n {database_name} -y')

        # Delete cluster
        self.cmd('az kusto cluster delete -n {cluster_name} -g {rg} -y')


if __name__ == '__main__':
    unittest.main()
